package edu.msoe.se1021.drawing.YOURNAME;
import java.awt.*;

/**
 * An AbstractShape represent an element to be displayed in a window.
 * AbstractShape-derived objects are maintained in a collection by the DrawingProgramApp class.
 */
public abstract class AbstractShape {
	public static final int LINE=0;
	public static final int RECTANGLE=1;
	public static final int ELLIPSE=2;
	public static final int UNKNOWN_SHAPE=-1;

	protected int x1; // starting x coordinate
	protected int y1; // starting y coordinate
	protected int x2; // ending x coordinate
	protected int y2; // ending y coordinate
	protected java.awt.Color color; // color to be used to draw this shape

	/**
	 * Create the String representation of this object
	 * Declaring this abstract method reminds every concrete shape
	 * class that it must implement this method. The format of the
	 * String should be consistent for every shape and of the format:
	 * <shape_type> <x1> <y1> <x2> <y2> <red> <green> <blue>
	 * For example, a blue line starting at coordinates 25, 50 and 
	 * extending to 100,200 would represent itself as follows:
	 * "0 25 50 100 200 0 0 255"
	 */
	@Override
	public abstract String toString();

	/**
	 * Constructor for initializing the common attributes of a shape
	 * 
	 * @param clr pen color to use when drawing this shape
	 * @param x1 starting x coordinate for this shape
	 * @param y1 starting y coordinate for this shape
	 * @param x2 ending x coordinate for this shape
	 * @param y2 ending y coordinate for this shape
	 */
	public AbstractShape(Color clr, int x1, int y1, int x2, int y2) {
		this.x1 = x1;
		this.y1 = y1;
		this.x2 = x2;
		this.y2 = y2;
		this.color = clr;
	}

	/**
	 * This abstract method defines the draw() method interface that every concrete shape
	 * class must implement
	 * @param g - the Graphics context to use for drawing
	 */
	public abstract void draw(Graphics g);

}