package edu.msoe.se1021.drawing.YOURNAME;
/**
 * Created on Feb 1, 2010
 *
 * DrawingProgramApp - an application for drawing simple shapes.
 * @author: hornick
 */
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.InputMismatchException;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Scanner;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JColorChooser;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicBorders;


/**
 * DrawingProgramApp is the main class for this application, and acts both
 * as the User Interface and as the application manager
 * @author: hornick
 */
@SuppressWarnings("serial")
public class DrawingProgramApp extends JFrame {
	private List<AbstractShape> shapes; // collection of concrete shapes (Lines, Ellipses, or Rectangles)

	private int width; // JFrame width (initial)
	private int height; // JFrame height (initial)
	private int xOrigin; // JFrame x-origin (initial)
	private int yOrigin; // JFrame y-origin (initial)

	//The JFrame's content pane is configured with a BorderLayout using only East, South, and Center 
	//The following three panels are placed in that layout.
	private JPanel jpSouth; // south JPanel; holds x,y labels indicating mouse coordinates
	private JPanel jpEast;  // east JPanel; holds radio buttons used to select the shape to be created
	private JPanel jpCenter; // center JPanel; the place where the shapes are drawn

	private JLabel xMouse; // current x mouse location in center JPanel-relative coordinates
	private JLabel yMouse; // current y mouse location in center JPanel-relative coordinates

	private Color penColor; // the pen color that is currently selected for created shapes
	private Color backgroundColor; // background color that is currently selected for the center panel
	private int shapeType; // the shape type currently selected via the radio buttons


	/**
	 * main method that starts this app going
	 * @param args - not used
	 */
	public static void main(String[] args) {
		new DrawingProgramApp(); // just call the constructor
	}

	/**
	 *  constructor; calls the methods that create the user interface
	 */
	private DrawingProgramApp() {
//TODO: create the collection to hold the shapes that will be created and drawn

		// initialize foreground (pen), background colors of the center panel
		penColor = Color.YELLOW;
		backgroundColor = Color.black;

		initUI(); // initialize basic UI
		createMenus(); // add the menus
		createShapeSelector(); // add the radio buttons for shape selection 
		createStatusDisplay(); // add the labels for mouse position display
		setVisible(true); // make the entire UI visible
	}

	/**
	 *  main UI JFrame initialization - size, title, layout, etc
	 */
	private void initUI() {
		// create the containing window and set its size etc
		setTitle("SE1021 Drawing Program"); 

		// initialize JFrame location, size
		width = 400;
		height = 300;
		xOrigin = yOrigin = 400;
		setSize(width, height);
		setLocation(xOrigin, yOrigin);
		
		setResizable(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Configure the content pane for a border layout
		Container canvas = getContentPane();
		canvas.setLayout(new BorderLayout(10,10));

		// create some panels to put in the various sections of the layout
		jpSouth = new JPanel( new FlowLayout(FlowLayout.LEFT));
		jpSouth.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
		jpSouth.setPreferredSize(new Dimension(100, 50));

		jpEast = new JPanel( new FlowLayout(FlowLayout.LEFT));
		jpEast.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
		jpEast.setPreferredSize(new Dimension(100, 50));

		// we use a DrawingPanel for the center panel in order to be able to override the paintComponent() method
		jpCenter = new DrawingPanel();
		jpCenter.setBackground(Color.BLACK);
//TODO: Create a MouseEventHandler and subscribe it to listen to mouse events from the center panel		

		// add the panels to the layout
		canvas.add(jpSouth, BorderLayout.SOUTH);
		canvas.add(jpEast, BorderLayout.EAST);
		canvas.add(jpCenter, BorderLayout.CENTER);
		
		jpEast.setVisible(true);
		jpSouth.setVisible(true);
		jpCenter.setVisible(true);
		
	}
	/**
	 * creates the menu and places it in the JFrame.
	 */
	private void createMenus() {
//TODO: 1) Create the nested MenuHandler that will respond to menu events

//TODO: 2) Create the menu bar, menus, and menu items.
// You need a File menu containing New, Open..., Save..., and Exit. Insert separators
// between New and Open..., and between Save... and Exit.
// You also need an Options menu containing Pen Color... and Background Color...

		
//TODO: 3) Subscribe the MenuHandler to listen for menu events.
	}

	/**
	 * Creates a set of radio buttons that specify the "active" shape;
	 * that is, whether the next item created is a Line, Ellipse, or Rectangle
	 */
	private void createShapeSelector() {
//TODO: 1) Create the  RadioButtonHandler that will respond to button events
		
//TODO: 2) Create the radio buttons used to select Line, Rectangle, or Ellipse.
// Place them in the East panel.
		
//TODO: 3) Subscribe the event handler to listen for radio button selection events


//TODO: 4) Add a ButtonGroup to manage the automatic 
// selection/deselection of radio buttons. 
	}

	/**
	 * Creates a couple of JLabels used to display the current
	 * mouse x,y coordinates with respect to the center panel.
	 */
	private void createStatusDisplay() {
//TODO: create the labels that will display the x,y location of the mouse
// Add these labels to the south panel.
	}



	/**
	 * create and add a Shape of the specified type and with the specified parameters to the collection
	 * @param type shape type - AbstractShape.ELLIPSE, etc
	 * @param c - Color of the shape
	 * @param x1 - starting x coordinate
	 * @param y1 - starting y coordinate
	 * @param x2 - ending x coordinate
	 * @param y2 - ending y coordinate
	 */
	private void createShape(int type, Color c, int x1, int y1, int x2, int y2 ) {
//TODO: Create a new shape of the specific type and add it to the collection of shapes.
// The specific shape depends on the current value contained in the shapeType attribute.
// Note that only shape creation is done here - not drawing; that's handled in the paintComponent() method
// of the DrawingPanel class (below). After you create the shape, the drawing of the shape will be 
// triggered by calling the repaint method:
		
		repaint(); // forces the shapes to be redrawn
	}

	/**
	 * updates the UI display of the current cursor location
	 * @param x x-coordinate of the cursor
	 * @param y y-coordinate of the cursor
	 */
	private void updateCursorLocation( int x, int y ) {
//TODO: update the labels indicating the mouse coordinates
	}

	/**
	 * clears the existing collection of shapes
	 */
	private void handleFileNewCommand() {
//TODO: erase the collection of shapes
		repaint();
	}
	
	
	/**
	 * posts the JColorChooser dialog to allow the user to specify the selected color
	 * @param cmd a String specifying which color to specify: 
	 * "Color" to set the penColor, "Background" to set the background
	 */
	private void handleColorCommand(String cmd) {
	}



	/**
	 * 	handle the file load command; posts the JFileChooser dialog and gets the filename, then delegates the actual
	 *  load() operation to the loadImage() method; posts errors thrown by that method
	 */
	private void handleFileLoadCommand() {
	}


	/**
	 * 	handle the file save command; posts the JFileChooser dialog and gets the filename, 
	 *  then delegates the actual save() operation to the saveImage() method; posts errors 
	 *  thrown by that method
	 */
	private void handleFileSaveCommand() {
	}

/* ****************************************************************************************
 * Action event handler for handling menu events
 * 
 */
	private class MenuHandler implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			// process commands from the menus
			String cmd = e.getActionCommand();
			if( cmd.equals("New") ) {
				handleFileNewCommand();
			}
			else if( cmd.equals("Load") ) {
				handleFileLoadCommand();
			}
			else if( cmd.equals("Save") ) {
				handleFileSaveCommand();
			}
			else if( cmd.equals("Color") ) {
				handleColorCommand("Color");
			}
			else if( cmd.equals("Background") ) {
				handleColorCommand("Background");
			}
			else if( cmd.equals("Exit") ) {
				System.exit(0);
			}
		}
	} // end MenuHandler inner class

/* ****************************************************************************************
 * Action event handler for handling radio button events
 * 
 */
	private class RadioButtonHandler implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
// TODO: process commands from the radio buttons
// Get the action command; set the shapeType attribute accordingly
		}
	} // end RadioButtonHandler inner class

/* ***************************************************************************************
 * Mouse event handler for handling mouse button and mouse motion events
 * 
 */
	private class MouseEventHandler implements MouseListener, MouseMotionListener {
		int xDown, yDown; // mouse coordinates when mouse button goes down
		int xUp, yUp; // mouse coordinates when mouse button goes up
//TODO: implement all interface methods - although most can have empty implementations.
// The important ones are the methods you'll use to track mouse motion (buttons pressed 
// or unpressed) in order to be able to update the mouse position in the south panel 
// via the updateCursorLocation() method (above).

// The other important methods
// are the ones for mouse button press and release; you'll use those methods to get the starting and ending
// points where the mouse button was pressed and subsequently released. When the mouse button is released,
// use these start and end points to create a new shape by calling the createShape() method (above).
	} // end MouseEvtHandler inner class

	
/* ***************************************************************************************
 * Inner class which defines a JPanel whose paintComponent() method can be
 * overridden in order to provide custom painting of the JPanel. This is needed
 * for us to be able to draw our shapes.
 */
	private class DrawingPanel extends JPanel {
		
		// constructor; just calls the JPanel constructor
		public DrawingPanel() {
			super();
		}

		/**
		 * paintComponent - override of the JPanel paintComponent() method. This is how paint
		 * message handling is done, rather than with explicit action handlers.
		 * Overriding the JPanel paintComponent() method gives us the opportunity to
		 * implement custom painting.
		 * 
		 * @param g graphics "context" - information about the pen color, font, etc etc
		 */
		@Override
		public void paintComponent(Graphics g){
			super.paintComponent(g); // invokes default painting for JPanel; must have this!

//TODO: Tell each shape in the collection to draw itself, passing on the graphics context
		}
	} // end DrawingPanel inner class
	
} // end DrawingProgramApp outer class
